import distutils
from distutils import util

from bson import json_util, ObjectId
from flask import Flask, request
from pymongo import MongoClient

from config.utils import Utils
from modele.contenu_dao import ContenuDao
from modele.demande_pubDao import DemandePubDao
from modele.encours_dao import EncoursDao
from modele.evaluation_dao import EvaluationDao
from modele.utilisateur_dao import UtilisateurDao

import json

from flask import Flask
from flask_cors import CORS, cross_origin

app = Flask(__name__)
cors = CORS(app)
app.config['CORS_HEADERS'] = 'Content-Type'

contenu_dao = ContenuDao()
utilisateur_dao = UtilisateurDao()
demande_pub_Dao = DemandePubDao()
evaluation_dao = EvaluationDao()
en_cours_dao = EncoursDao()

client = MongoClient('localhost', 27017)

base_donnees = client.squelette


## *********************************************************************************************************************
## Routage utilisateur : 1 post, 1 get, 2 patch
@app.route('/newUser', methods=['POST'])
@cross_origin()
def creer_utilisateur():
    utilisateur_dao.ajouter_utilisateur(request.json["login"])
    return "Utilisateur créé"


@app.route('/checkUser', methods=['GET'])
@cross_origin()
def verifier_utilisateur():
    return str(utilisateur_dao.presence_utilisateur(request.args.get("login")))


@app.route('/changePassword', methods=['PATCH'])
@cross_origin()
def changer_mdp():
    pass


@app.route('/ajouterTiers', methods=['PATCH'])
@cross_origin()
def ajouter_tiers():
    id_tiers = json.loads(request.json["id_tiers"])
    contenu_dao.creer_tiers(request.json["auteur"], id_tiers["$oid"], request.json["version"])
    return "Tiers ajouté"


## *********************************************************************************************************************
## Routage contenus : beaucoup de fonctions
@app.route("/addContenu", methods=['POST'])
@cross_origin()
def ajouter_contenu():
    auteur = request.json["auteur"]
    contenu_a_ecrire = request.json["contenu_a_ecrire"]
    contenu_dao.ajouter_contenu(auteur, contenu_a_ecrire)
    return "Contenu ajouré"


@app.route("/getContenu/<id>", methods=['GET'])
@cross_origin()
def get_contenu_par_id(id):
    return json.dumps(list(contenu_dao.get_contenu_par_id(int(id))), default=json_util.default)


@app.route("/getContenuPanier/<auteur>", methods=['GET'])
@cross_origin()
def get_contenu_panier(auteur):
    auteur = auteur.replace("_", " ")
    return json.dumps(contenu_dao.get_contenus(auteur, request.args.get("perso"), request.args.get("tiers"),
                                               request.args.get("enCours")),
                      default=json_util.default)


@app.route("/getContenuBanque/<auteur>", methods=['GET'])
@cross_origin()
def get_contenu_banque(auteur):
    auteur = auteur.replace("_", " ")
    return json.dumps(list(contenu_dao.get_contenus_non_auteur(auteur)),
                      default=json_util.default)


@app.route("/deletePerso/<id_perso>", methods=['DELETE'])
@cross_origin()
def supprimer_perso(id_perso):
    id_perso = json.loads(id_perso)
    contenu_dao.supprimer_contenu(id_perso["$oid"])
    return "Contenu supprimé"


@app.route("/deleteTiers", methods=['DELETE'])
@cross_origin()
def supprimer_tiers():
    id_tiers = json.loads(request.args.get("id_tiers"))

    version = contenu_dao.get_version_tiers_avec_id(id_tiers["$oid"], request.args.get('auteur'))
    contenu_dao.supprimer_tiers(id_tiers["$oid"], request.args.get('auteur'), version)
    return "Contenu Tiers supprimé"


@app.route("/newContenu", methods=['POST'])
@cross_origin()
def creer_nouveau_contenu():
    id_demande_pub = json.loads(request.json["id_demande_pub"])
    contenu_dao.creer_nouveau_contenu(id_demande_pub["$oid"], request.json["auteur"])
    return "Nouveau contenu créé"


@app.route("/majContenu", methods=['PATCH'])
@cross_origin()
def maj_contenu():
    id_contenu = json.loads(request.json["id_contenu"])
    if request.json["auteur"] == "":
        auteur = None
    else:
        auteur = request.json["auteur"]
    contenu_dao.maj_contenu(id_contenu["$oid"], auteur)
    return "Le contenu a été mis à jour"


@app.route("/gererDemandePub", methods=['PATCH'])
@cross_origin()
def gerer_demande_pub():
    id_publication = json.loads(request.json["id_publication"])
    demande_pub_Dao.gerer_demande_pub(id_publication["$oid"], request.json["reponse"], request.json["auteur"])
    return "Réponse à la publication gérée"


@app.route("/allContenus", methods=['GET'])
@cross_origin()
def get_tous_les_contenus():
    return json.dumps(list(contenu_dao.get_tous_les_contenus()), default=json_util.default)


@app.route("/isContenuAnonyme/<id_contenu>", methods=['GET'])
@cross_origin()
def is_contenu_anonyme(id_contenu):
    return str(contenu_dao.is_anonyme(id_contenu))


@app.route("/isMajTiersDisponible/<auteur>", methods=['GET'])
@cross_origin()
def is_maj_tiers_disponible(auteur):
    return str(contenu_dao.is_tiers_maj(auteur))


@app.route("/listeMajTiers/<auteur>", methods=['GET'])
@cross_origin()
def liste_maj_tiers(auteur):
    return json.dumps(contenu_dao.liste_tiers_maj(auteur), default=json_util.default)


@app.route("/gererMajTiers", methods=['PATCH'])
@cross_origin()
def gerer_maj_tiers():
    id_tiers = json.loads(request.json["id_tiers"])

    if request.json["reponse"] == 'o':
        refuse = False
    else:
        refuse = True

    version = contenu_dao.get_version_tiers_avec_id(id_tiers["$oid"], request.json["auteur"])

    contenu_dao.maj_tiers(id_tiers["$oid"], int(version) + 1, request.json["auteur"], refuse)
    return "Tiers géré"


@app.route("/getTexteContenu/<id_json>", methods=['GET'])
@cross_origin()
def get_texte_contenu(id_json):
    id_contenu = json.loads(id_json)
    return Utils().recuperer_contenu_fichier(id_contenu["$oid"])


@app.route("/changerTexte/<id_contenu>", methods=['PATCH'])
@cross_origin()
def changer_texte_contenu(id_contenu):

    if len(id_contenu) % 12 == 0:
        id_contenu = ObjectId(id_contenu)
    else:
        tmp = json.loads(id_contenu)
        id_contenu = tmp["$oid"]

    Utils().changer_texte_fichier(id_contenu, request.json["nouveau_texte"])
    return "Texte Maj"


## *********************************************************************************************************************
## Gestion des Contenus en Cours

@app.route("/getTexteContenuEnCours/<id_json>", methods=['GET'])
@cross_origin()
def get_texte_contenu_en_cours(id_json):
    if len(id_json) % 12 == 0:

        id_du_contenu = ObjectId(id_json)
    else:
        tmp = json.loads(id_json)
        id_du_contenu = tmp["$oid"]

    return Utils().recuperer_contenu_fichier(id_du_contenu, True)


@app.route("/getEnCours/<id_en_cours>", methods=['GET'])
@cross_origin()
def get_contenu_en_cours(id_en_cours):
    return json.dumps(list(en_cours_dao.get_contenu_en_cours(id_en_cours)), default=json_util.default)


@app.route("/getAuteurOrigineEnCours/<id_en_cours>", methods=['GET'])
@cross_origin()
def get_auteur_otigine_contenu_en_cours(id_en_cours):
    return json.dumps(list(en_cours_dao.get_auteur_origine_contenu_en_cours(id_en_cours)), default=json_util.default)


@app.route("/isContenuDemandePublication/<id_en_cours>/<auteur>", methods=['GET'])
@cross_origin()
def is_contenu_demande_publication(id_en_cours, auteur):
    return str(en_cours_dao.is_en_cours_demande(id_en_cours, auteur))


@app.route("/supprimerEnCours/<id_reference>", methods=['DELETE'])
@cross_origin()
def supprimer_en_cours(id_reference):
    id_contenu = json.loads(id_reference)
    en_cours_dao.supprimer_contenu_en_cours(id_contenu["$oid"])
    return "Contenu En Cours supprimé"


@app.route("/supprimerEnCoursParId/<id_en_cours>", methods=['DELETE'])
@cross_origin()
def supprimer_en_cours_par_id(id_en_cours):
    id_contenu = json.loads(id_en_cours)
    en_cours_dao.supprimer_contenu_en_cours_via_id(id_contenu["$oid"])
    return "Contenu En Cours supprimé"


@app.route("/newEnCours", methods=['POST'])
@cross_origin()
def creer_en_cours():
    id_contenu = json.loads(request.json["id_contenu"])
    en_cours_dao.creer_en_cours(request.json['auteur'], id_contenu["$oid"], request.json['version'])
    return en_cours_dao.recuperer_dernier_id()


@app.route("/newEnCoursRefuse", methods=['POST'])
@cross_origin()
def creer_en_cours_refuse():
    id_contenu = json.loads(request.json["id_contenu"])

    en_cours_dao.creer_en_cours_refuse(request.json['auteur'], id_contenu["$oid"], request.json['version'])
    return "En Cours Refuse créé"


## *********************************************************************************************************************
## Gestion des Publications

@app.route("/demandePublication", methods=['POST'])
@cross_origin()
def demande_de_publication():
    id_contenu = json.loads(request.json["id_contenu"])
    id_contenu_en_cours = json.loads(request.json["id_contenu_en_cours"])

    auteur = en_cours_dao.get_auteur_origine_contenu_en_cours(id_contenu_en_cours["$oid"])
    demande_pub_Dao.creer_demande_publication(id_contenu["$oid"], id_contenu_en_cours["$oid"],
                                              request.json['auteur_en_cours'], auteur, request.json["version"])
    return "Demande de publication créée"


@app.route("/demandePublication/<auteur>/<emise>", methods=['GET'])
@cross_origin()
def get_demandes_pub(auteur, emise):
    return json.dumps(list(demande_pub_Dao.get_demandes_pub(auteur, distutils.util.strtobool(emise))),
                      default=json_util.default)


@app.route("/demandePublication/<id_publication>", methods=['DELETE'])
@cross_origin()
def supprimer_demande_pub(id_publication):
    id_publication = json.loads(id_publication)

    demande_pub_Dao.supprimer_publication(id_publication["$oid"])
    return "Demande de publication supprimée"


## *********************************************************************************************************************
## Gestion des Evaluations

@app.route("/newEval", methods=['POST'])
@cross_origin()
def creer_evaluation():
    evaluation_dao.creer_evaluation(request.args.get('contenus'), request.args.get('auteur'))
    return "evaluation créée"


@app.route("/getEval/<auteur>/<id_evaluation>", methods=['GET'])
@cross_origin()
def get_evaluation(auteur, id_evaluation):
    return json.dumps(list(evaluation_dao.get_evaluations(auteur, id_evaluation)), default=json_util.default)


@app.route("/getEval/<auteur>", methods=['GET'])
@cross_origin()
def get_all_evaluations(auteur):
    return json.dumps(list(evaluation_dao.get_evaluations(auteur, 0)), default=json_util.default)


@app.route("/deleteEval", methods=['DELETE'])
@cross_origin()
def supprimer_evaluation(id_evaluation):
    evaluation_dao.supprimer_evaluation(id_evaluation)


if __name__ == '__main__':
    app.run()
