from bson import ObjectId
from flask_pymongo import PyMongo

from pymongo import MongoClient
from random import randrange
from datetime import datetime
import os
# from UtilsGit import UtilsGit
from pathlib import Path
import string

from config.gitUtils import GitUtils
from modele.encours_dao import EncoursDao


class ContenuDao:
    client = MongoClient('localhost', 27017)
    chemin = "/home/passerat/Stage/flaskProject/"
    gitUtils = GitUtils()

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    ## Fonction permettant d'ajouter un contenu dans le panier de l'auteur
    # @param l'auteur en cours
    # @param nom_fichier le nom du fichier associé au contenu
    # @param commit le 'hash' du commit associé (récupéré via git)
    def ajouter_contenu(self, auteur, contenu_a_ecrire):

        # créer la métadonnée dans la base mongo
        self.base_donnees.contenu.insert_one(
            {"auteur": auteur, "note": randrange(0, 10, 1),
             "fichier": "",
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "popularite": 0, "commit": ""},
             "tags_banque_fixes": ["tag1", "tag2", "tag3", "tag4", "tag5"],
             "tags_banque_libres": ["tagl1", "tagl2", "tagl3", "tagl4", "tagl5"], "type": " ", "source": " "})

        # créer un fichier (au bon endroit)
        # récupérer l'id mongo (pour nommer le répertoire)
        never_shall_we_die = self.base_donnees.contenu.find().sort("_id", -1).limit(1)

        record = next(never_shall_we_die, None)
        nom_contenu = str(record.get("_id"))

        chemin_fichier = self.chemin + "/" + nom_contenu + "/"
        Path(chemin_fichier).mkdir(parents=True, exist_ok=True)

        myFile = open(chemin_fichier + nom_contenu, "w")
        # y ajouter le texte du contenu
        myFile.write(contenu_a_ecrire)
        myFile.close()

        # mettre a jour le chemin du contenu
        self.base_donnees.contenu.update_one({"_id": record.get("_id")},
                                             {"$set": {"fichier": nom_contenu + "/" + nom_contenu}})

        # initialiser le git et récupérer le commit
        commit = self.gitUtils.commit(chemin_fichier, nom_contenu, "version" + str(1))

        #  puis le mettre a jour dans le contenu
        self.base_donnees.contenu.update_one({"_id": record.get("_id")}, {"$set": {"commit": commit}})

    ## recherche d'un contenu par id (pour affichage de son contenu. charge la version courante
    # @param id_contenu l'id du contenu recherché
    def get_contenu_par_id(self, id_contenu):
        return self.base_donnees.contenu.find({"_id": ObjectId(id_contenu)})

    ## recherche d'un contenu par id et une version précise
    # @param id l'id du contenu recherché
    # @param version la version du contenu recherché
    def get_contenu_avec_id_et_version(self, id_contenu, version):

        # a décommenter quand git sera actif
        # Utils.changer_version(idContenu, version, "")
        return self.base_donnees.contenu.find({"_id": ObjectId(id_contenu["contenu"])})
        # return Utils.contenu.find_one({"$and": [
        #     {"id": idContenu},
        #     {"$or": [{"versionEnCours.numero": version}, {"historique.numero": version}]}
        # ]
        # })

    ## recherche des contenus pour un auteur donné (son 'panier')
    # @param auteur l'auteur des contenus
    # @param perso true si contenus persos a charger, false sinon
    # @param tiers true si contenus tiers a charger, false sinon
    # @param en_vours true si contenus en cours a charger, false sinon
    # @return tous les contenus présents dans le panier de l'auteur
    def get_contenus(self, auteur, perso, tiers, en_cours):

        contenus = list()

        if tiers:
            id_tiers = self.get_contenus_tiers(auteur)
            for (id, version) in id_tiers:
                curseur_tiers = self.get_contenu_avec_id_et_version(id, version)
                for contenu in curseur_tiers:
                    contenu["provenance"] = "tiers"
                    contenu["version"] = version
                    contenus.append(contenu)

        if perso:
            curseur_perso = self.get_contenus_persos(auteur)
            for contenu in curseur_perso:
                contenu["provenance"] = "perso"
                contenus.append(contenu)

        if en_cours:
            curseur_en_cours = EncoursDao().get_contenus_en_cours_auteur(auteur)
            for contenu in curseur_en_cours:
                contenu["provenance"] = "en-cours"
                contenus.append(contenu)
            # on garde le développement de l'évaliation pour plus tard
            # return contenus_persos + contenus_en_cours + contenus_tiers
        return contenus

    ## FOnction permettant de récupérer les contenus tiers d'un auteur
    # @param auteur l'auteur en question
    def get_contenus_tiers(self, auteur):
        return self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"]

    ## Fonction permettant de charger tous les contenus 'publiés' n'appartenant pas à auteur (pour afficher sa banque)
    # @param auteur l'auteur en cours
    # @return tous les contenus non présents dans le panier de l'auteur
    def get_contenus_non_auteur(self, auteur):
        contenus_tiers = self.get_contenus_tiers(auteur)
        tiers_auteur = list()
        for tier in contenus_tiers:
            tiers_auteur.append(ObjectId(tier[0]["contenu"]))

        return self.base_donnees.contenu.find({"$and": [{"auteur": {"$ne": auteur}}, {"_id": {"$nin": tiers_auteur}}]})

    ## FOnction de création d'un tiers
    # @param auteur l'auteur en cours
    # @param id_contenu le contenu à ajouter en tiers
    # @param version la version dans laquelle le contenu a été pris
    def creer_tiers(self, auteur, id_contenu, version):
        self.base_donnees.utilisateur.update({"nom": auteur}, {
            "$push": {
                "tiers": [{"contenu": ObjectId(id_contenu)}, {"version": version}]}})

    ## FOnction permettant de récupérer les contenus persos d'un auteur
    # auteur l'auteur en question
    def get_contenus_persos(self, auteur):
        return self.base_donnees.contenu.find({"auteur": auteur})

    ## l'auteur 'renie' son contenu, qui reste disponible dans le panier de ceux qui le possédent
    # @param auteur l'id du contenu à renier
    def dissocier_contenu(self, id_contenu):
        self.base_donnees.contenu.update({"_id": ObjectId(id_contenu)}, {"$set": {"auteur": "Aucun"}})

    ## l'auteur 'renie' son contenu, mais vu que personne d'autre ne l'a il est définitivement supprimé
    # @param auteur l'id du contenu à supprimer
    def supprimer_contenu(self, id_contenu):

        if self.existe_en_tiers(id_contenu):
            self.dissocier_contenu(id_contenu)
        else:
            self.base_donnees.contenu.delete_one({"_id": ObjectId(id_contenu)})

    ## l'auteur 'renie' un tiers (suppression définitive)
    # @param auteur l'id du contenu à supprimer
    def supprimer_tiers(self, id_tiers, auteur, version):
        self.base_donnees.utilisateur.update({"nom": auteur},
                                             {"$pull": {
                                                 "tiers": [{"contenu": ObjectId(id_tiers)}, {"version": version}]}})

    ## fonction qui permet de savoir si  au moins un auteur possède un certain contenu dans ses tiers
    # @param auteur l'id du contenu à 'tester'
    def existe_en_tiers(self, id_contenu):
        return self.base_donnees.utilisateur.find(
            {"tiers": {"$elemMatch": {"$elemMatch": {"contenu": ObjectId(id_contenu)}}}}).count() != 0

    ## fonction qui permet de créer un contenu à partir d'une contenu en cours
    # @param chemin le chemin du contenu
    # @param auteur_en_cours l'auteur du contenu en cours
    # @param id_contenu_en_cours l'id du contenu en cours
    def creer_nouveau_contenu(self, id_demande_publication, auteur_en_cours):

        # hard codé pour l'exemple, sera adaptatif dans le futur
        max_id = 3

        id_contenu = modele.DemandePubDao().get_id_contenu_associe_a_une_publication(id_demande_publication)
        id_contenu_en_cours = modele.DemandePubDao().get_id_contenu_en_cours_associe_a_une_publication(id_demande_publication)

        chemin = self.get_chemin_du_contenu(id_contenu)

        dossierFichier = "dossierContenu" + str(max_id) + "/"
        cheminFichier = self.chemin + dossierFichier
        nomFichier = chemin[chemin.rfind("/") + 1:]

        # Path(cheminFichier).mkdir(parents=True, exist_ok=True)

        self.base_donnees.contenu.insert_one(
            {"auteur": auteur_en_cours, "note": randrange(0, 10, 1),
             "fichier": "dossierContenu" + str(max_id) + "/Contenu" + str(max_id),
             "versionEnCours": {"numero": 1, "date": datetime.now().strftime("%d/%m/%y"),
                                "popularite": 0, "commit": ""},
             "tags_fixes": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1",
                            "5": "tag1"},
             "tags": {"1": "tag1", "2": "tag1", "3": "tag1", "4": "tag1", "5": "tag1"}})

        modele.DemandePubDao().supprimer_publication(id_demande_publication)

    ## récupère le chemin d'un contenu
    # @param id_contenu l'id du contenu recherché
    # @return le chemin désiré
    def get_chemin_du_contenu(self, id_contenu):

        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)
        return self.chemin + self.base_donnees.contenu.find_one({"_id": id_contenu})["fichier"]

    ## récupère la version en cours du contenu
    def get_version_courante(self, id_contenu):
        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)
        return self.base_donnees.contenu.find_one({"_id": id_contenu})["versionEnCours"]["numero"]

    ## récupère la version d'un contenu mis en tiers (ou 0 si ce contenu n'est pas sossédé en tiers)
    # @param auteur l'auteur concerné
    # @param id_contenu le contenu recherché
    # @return la version du tiers id_contenu (ou 0)
    def get_version_tiers_avec_id(self, id_contenu, auteur):
        tiers = (self.base_donnees.utilisateur.find_one({"nom": auteur})["tiers"])
        for i in range(len(tiers)):
            tmp = tiers[i]
            if tmp[0]["contenu"] == ObjectId(id_contenu):
                return int(tmp[1]["version"])
        return 0

    ## mise à jour d'un contenu
    # @param idContenu l'id du contenu à mettre à jour
    # @param nomBranche le nom de la branche
    # @param auteur l'auteur du contenu à mettre à jour
    def maj_contenu(self, id_contenu, auteur=None):

        chemin = self.get_chemin_du_contenu(id_contenu)

        repertoire = chemin[:chemin.rfind("/") + 1]
        version = int(self.get_version_courante(id_contenu)) + 1

        # Mise à jour du répertoire git
        # idCommit = UtilsGit.mergeVersion(repertoire, nomBranche)

        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)

        # Mise à jour des métadonnées - ok
        version_en_cours = self.base_donnees.contenu.find_one({"_id": id_contenu})["versionEnCours"]
        self.base_donnees.contenu.update_one({"_id": id_contenu}, {"$push": {"historique": version_en_cours}})
        self.base_donnees.contenu.update_one({"_id": id_contenu}, {
            "$set": {"versionEnCours": {"numero": version, "date": datetime.now().strftime("%d/%m/%y"), "popularite": 0,
                                        "commit": ""}}})
        if auteur is not None:
            self.base_donnees.contenu.update_one({"_id": id_contenu}, {
                "$set": {"auteur": auteur}})

    ## lecture d'un contenu (version basique pour les tests)
    # @param id l'id du contenu à consulter
    # @param auteur l'auteur souhaitant consulter le contenu
    def consulter_un_contenu(self, id, auteur):
        # Créer un répertoire au nom de l'auteur (ou l'utiliser si il existe déja)
        auteur = auteur.lower().replace(" ", "_")
        repertoire_auteur = self.chemin + auteur + "/"
        Path(repertoire_auteur).mkdir(parents=True, exist_ok=True)
        # Copier le fichier lié au contenu dans ce répertoire
        chemin = self.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("cp '" + chemin + "'  '" + repertoire_auteur + nom_fichier + "'")
        # self.affichage(chemin)



    ## après fin de consultation, destruction du fichier temporaire généré
    # @param id l'id du contenu consulté
    # @param auteur l'auteur auant effectué la consultation
    def detruire_fichier_tmp(self, id, auteur):
        auteur = auteur.lower().replace(" ", "_") + "/"
        chemin = self.get_chemin_du_contenu(id)
        nom_fichier = chemin[chemin.rfind("/") + 1:]
        os.system("rm '" + self.chemin + auteur + nom_fichier + "'")

    ## Fonction permettant de déterminer si un contenu est "anonyme"
    # @param idCOntenu le contenu à tester
    # @return booléen true.false si le contenu est anonyme ou pas
    def is_anonyme(self, idContenu):
        # print("Is anonyme : ", Utils.contenu.find_one({"id": idContenu})["auteur"])
        return self.base_donnees.contenu.find_one({"id": idContenu})["auteur"] == "Aucun"

    ## Récupérer tous les contenus
    def get_tous_les_contenus(self):
        return self.base_donnees.contenu.find()

    # FOnction permettant de déterminer si un certain contenu fait partie des tiers d'un auteur
    # @param auteur l'auteur concerné
    # @param idContenu le contenu à 'tester'
    # @return booléen true.false si le contenu est chez l'auteur ou pas
    def is_tiers_auteur(self, auteur, idContenu):
        return self.get_version_tiers_avec_id(ObjectId(idContenu), auteur) != 0

    ## mettre à jour un contenu 'tiers'
    # @param auteur l'auteur concerné
    # @param id_favori le favori à maj
    # @param nouvelle_version le numero de la nouvelle version
    # @param refuse indique si la mise a jour est acceptee ou refusee
    def maj_tiers(self, id_tiers, nouvelle_version, auteur, refuse=False):
        if not refuse:

            # on vire le précédent
            self.base_donnees.utilisateur.update({"nom": auteur},
                                                 {"$pull": {
                                                     "tiers": [{"contenu": ObjectId(id_tiers)},
                                                               {"version": nouvelle_version - 1}]}})

            # et on ajoute la version maj
            self.base_donnees.utilisateur.update({"nom": auteur},
                                                 {"$push": {
                                                     "tiers": [{"contenu": ObjectId(id_tiers)},
                                                               {"version": nouvelle_version}]}})
        else:
            self.base_donnees.utilisateur.update({"nom": auteur},
                                                 {"$push": {
                                                     "majRejetees": [{"contenu": ObjectId(id_tiers),
                                                                      "version": nouvelle_version}]}})

    ## Permet de savoir si au moins un tiers est mettable à jour pour un auteur
    # @param auteur l'auteur concerné
    # @return un booléen indiquant si on a des mises à jour faisables
    def is_tiers_maj(self, auteur):
        # on cherche toutes les versions en cours dans les tiers et on compare avec celles de la banque

        # On réfcupère tous les ties d'un auteur
        tiers = self.get_contenus_tiers(auteur)

        for (id, version) in tiers:
            versionCourante = self.get_version_courante(id["contenu"])
            if version["version"] < versionCourante:
                return True

        return False

    ## Permet de savoir si au moins un tiers est mettable à jour pour un auteur
    # @param auteur l'auteur concerné
    # @return la liste des tiers mettables à jour
    def liste_tiers_maj(self, auteur):
        liste = list()
        # On réfcupère tous les ties d'un auteur
        tiers = self.get_contenus_tiers(auteur)

        for (id, version) in tiers:
            versionCourante = self.get_version_courante(id["contenu"])
            if version["version"] < versionCourante:
                liste += self.get_contenu_avec_id_et_version(id, version)

        return liste

    def get_commit_pour_une_version(self, idContenu, version):
        if version == self.get_version_courante(idContenu):
            courante = self.base_donnees.contenu.find_one({"id": idContenu})["versionEnCours"]
            return courante["commit"]
        else:
            historique = self.base_donnees.contenu.find_one({"id": idContenu})["historique"]
            for elt in historique:
                if elt["numero"] == version:
                    return elt["commit"]
