from pymongo import MongoClient
from datetime import datetime
from bson import ObjectId




class DemandePubDao:
    client = MongoClient('localhost', 27017)

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    ## Crétion d'une demande de publication
    # @param idCOntenu l'id du contenu d'origine
    # @param auteur l'auteur du contenu d'origine
    # @param idContenuEnCours l'id du contenu en cours
    # @param auteurEnCours l'auteur du contenu en cours

    def creer_demande_publication(self, id_contenu, id_contenu_en_cours, auteur_en_cours, auteur, version_en_cours):

        self.base_donnees.demandesPub.insert_one(
            {"date": datetime.now().strftime("%d/%m/%y"), "auteurDemande": auteur_en_cours,
             "auteurContenu": auteur, "idContenu": ObjectId(id_contenu),
             "idContenuEnCours": ObjectId(id_contenu_en_cours), "version_en_cours": version_en_cours,
             "etat": "enCours"})

    ## Fonction permettant de gérer une demande de publication
    # @param idPublication l'id de la demande de publication
    # @param reponse la réponse donnée par l'auteur d'origine
    # @param auteur l'auteur d'origine
    def gerer_demande_pub(self, idPublication, reponse, auteurEnCours):
        if reponse == 'o':
            # branche = self.get_branche_contenu_en_cours(id_en_cours)

            # on récupère l'id du contenu d'origine et l'id en cours
            id_contenu = self.get_id_contenu_associe_a_une_publication(idPublication)
            id_contenu_en_cours = self.get_id_contenu_en_cours_associe_a_une_publication(idPublication)

            # et on met à jour le contenu d'id 'id_contenu' (sans en modifier l'auteur)
            ContenuDao().maj_contenu(id_contenu, None)

            # on notifie au 'demandeur' que sa demande de publication est acceptée
            self.base_donnees.demandesPub.update_one({"_id": ObjectId(idPublication)}, {"$set": {"etat": "accepté"}})

            version = modele.ContenuDao().get_version_courante(id_contenu)

            # On met à jour le tiers (chez le 'demandeur' de la publication)
            modele.ContenuDao().maj_tiers(id_contenu, version, auteurEnCours)
            modele.EncoursDao().supprimer_contenu_en_cours(id_contenu_en_cours)
        else:
            # on change l'état de la demande de publication
            self.base_donnees.demandesPub.update_one({"_id": ObjectId(idPublication)}, {"$set": {"etat": "refus"}})

    ## Supprimer une demande de publication
    # @param idPub l'id de la demande de publication

    def supprimer_publication(self, idPub):
        self.base_donnees.demandesPub.delete_one({"_id": ObjectId(idPub)})

    ## Récupérer l'auteur d'une demande de publication
    # @param idPub l'id de la demande de publication
    def get_auteur_demande_publication(self, idDdePublication):
        return self.base_donnees.demandesPub.find_one({"_id": ObjectId(idDdePublication)})["auteurDemande"]

    ## Récupérer lid du contenu (perso ou tiers) associé à une demande de publication
    # @param idDdePublication l'id de la demande de publication
    def get_id_contenu_associe_a_une_publication(self, idDdePublication):
        return self.base_donnees.demandesPub.find_one({"_id": ObjectId(idDdePublication)})["idContenu"]

    ## Récupérer lid du contenu en cours associé à une demande de publication
    # @param idDdePublication l'id de la demande de publication
    def get_id_contenu_en_cours_associe_a_une_publication(self, idDdePublication):
        return self.base_donnees.demandesPub.find_one({"_id": ObjectId(idDdePublication)})["idContenuEnCours"]

    ## Récupérer les demandes de publication
    # @param auteur l'auteur des demandes
    # @param emises si true, récupère les demandes emises; les demandes recues sinon
    # @return les demandes de publications idoines
    def get_demandes_pub(self, auteur, emises=True):
        if emises:
            return self.base_donnees.demandesPub.find({"auteurDemande": auteur})
        else:
            return self.base_donnees.demandesPub.find({"auteurContenu": auteur})
