from pathlib import Path

from bson import ObjectId
from pymongo import MongoClient
from datetime import datetime

from config.gitUtils import GitUtils
from config.utils import Utils


class EncoursDao:
    client = MongoClient('localhost', 27017)
    chemin = "/home/passerat/Stage/flaskProject/"
    gitUtils = GitUtils()
    utils = Utils()

    ## extraction d'une base de données. Si elle n'existe pas, elle esst créée
    base_donnees = client.squelette

    ## Fonction qui permet de savoir si un contenu en cours à fait l'objet d'une demande de publication
    # @param idEnCours l'id du contenu en cours
    # @param l'auteur l'auteur potentiel de la demande de publication
    def is_en_cours_demande(self, idEnCours, auteur):
        return self.base_donnees.demandesPub.find({"id": idEnCours} and {"auteurDemande": auteur}).count() != 0

    ## récupérer l'auteur initial d'un contenu en cours (peut être l'auteur effectuant la modification ou non)
    # @param idContenuEnCours l'id du contenu en cours de modification
    # @return l'auteur intial du contenu en cours
    def get_auteur_origine_contenu_en_cours(self, id_contenu_en_cours):
        id_contenu = self.base_donnees.encours.find_one({"_id": ObjectId(id_contenu_en_cours)})["reference"]
        return self.base_donnees.contenu.find_one({"_id": id_contenu})["auteur"]

    ## récupérer l'id initial d'un contenu en cours (peut être l'auteur effectuant la modification ou non)
    # @param idContenuEnCours l'id du contenu en cours de modification
    # @return l'id intial du contenu en cours
    def get_id_origine_contenu_en_cours(self, id_contenu_en_cours):
        return self.base_donnees.encours.find_one({"_id": ObjectId(id_contenu_en_cours)})["reference"]

    ## permet de récupérer tous les contenus en cours pour un auteur
    # @param auteur le nom de l'auteur
    # @return les contenus en cours de l'auteur
    def get_contenus_en_cours_auteur(self, auteur):
        return self.base_donnees.encours.find({"auteur": auteur})

    ## récupérer un contenu en cours
    # @param idContenuEnCours l'id du contenu en cours de modification
    # @return le contenu en cours
    def get_contenu_en_cours(self, id_contenu_en_cours):
        # id_contenu = Utils.getIdContenuAssocieAUnEnCours(idContenuEnCours)
        return self.base_donnees.encours.find_one({"idContenuEC": id_contenu_en_cours})

    ## supprimer un contenu en cours
    # @param idContenu id du contenu  de la reference a supprimer
    def supprimer_contenu_en_cours(self, idContenuEnCours):
        self.base_donnees.encours.delete_one({"reference": ObjectId(idContenuEnCours)})

    ## supprimer un contenu en cours avec l'id
    # @param idContenuEnCours id du contenu a supprimer
    def supprimer_contenu_en_cours_via_id(self, idContenuEnCours):
        self.base_donnees.encours.delete_one({"_id": ObjectId(idContenuEnCours)})

    ## création d'un contenu en cours
    # @param version la version du contenu d'origine
    # @param nomBranche -> plus utile.
    def creer_en_cours(self, auteur, id_contenu, version):
        self.base_donnees.encours.insert_one(
            {"date": datetime.now().strftime("%d/%m/%y"), "fichier": "", "auteur": auteur,
             "reference": ObjectId(id_contenu), "version": version})

        id = self.recuperer_dernier_id()

        # Deux choix -> création d'une branche dans le répertoire idContenu / idContenu

        # chemin = ContenuDao().get_chemin_du_contenu(id_contenu)
        #
        # self.base_donnees.encours.update_one({"_id": id},
        #                                      {"$set": {"branche": id}})
        #
        # GitUtils().changerVersionBranche(chemin, id)

        # Ou création d'un autre répertoire id_en_cours / id_en_cours

        chemin_fichier = self.chemin + "en_cours/"

        Path(chemin_fichier).mkdir(parents=True, exist_ok=True)

        myFile = open(chemin_fichier + id, "w")
        # y ajouter le texte du contenu
        myFile.write(self.utils.recuperer_contenu_fichier(id_contenu))
        myFile.close()

        # mettre a jour le chemin du contenu
        self.base_donnees.encours.update_one({"_id": ObjectId(id)},
                                             {"$set": {"fichier": "en_cours/" + id}})

        # initialiser le git et récupérer le commit
        # self.gitUtils.commit(chemin_fichier, id, "version" + str(version))

    ## création d'un contenu en cours, dans l'état refuse
    # @param version la version du contenu d'origine
    # @param nomBranche -> plus utile.
    def creer_en_cours_refuse(self, auteur, idContenu, version):
        self.base_donnees.encours.insert_one(
            {"date": datetime.now().strftime("%d/%m/%y"), "auteur": auteur,
             "reference": ObjectId(idContenu), "version": version, "etat": "refuse"})

    def recuperer_dernier_id(self):
        dernier_id_insere = self.base_donnees.encours.find().sort("_id", -1).limit(1)

        record = next(dernier_id_insere, None)
        return str(record.get("_id"))

    def get_chemin_contenu_en_cours(self, id_contenu):
        if type('id_contenu') == str:
            id_contenu = ObjectId(id_contenu)

        return self.chemin + self.base_donnees.encours.find_one({"_id": id_contenu})["fichier"]
